package cn.sm1234.test;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.domain.Sort.Order;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.test.annotation.Rollback;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.transaction.annotation.Transactional;

import cn.sm1234.Application;
import cn.sm1234.dao.EmpRepository;
import cn.sm1234.domain.Emp;

@RunWith(SpringJUnit4ClassRunner.class)
@SpringBootTest(classes = Application.class)
public class EmpRepositoryTest {

	@Resource
	private EmpRepository empRepository;

	/*
	 * @Test public void testSave(){ Emp emp = new Emp(); emp.setName("王五");
	 * emp.setGender("女"); emp.setTelephone("1346666677777");
	 * emp.setAddress("广州番禺");
	 * 
	 * //empRepository.save(emp); }
	 */

	@Test
	public void testFindByName() {
		List<Emp> list = empRepository.findByName("张三");
		for (Emp emp : list) {
			System.err.println(emp);
		}
	}

	@Test
	public void testFindByNameAndGender() {
		List<Emp> list = empRepository.findByNameAndGender("李四", "男");
		for (Emp emp : list) {
			System.err.println(emp);
		}
	}

	/*
	 * 如果是Like查询，那么需要加上模糊查询的关键词 %： 匹配任意个字符 _ : 匹配一个字符
	 */
	@Test
	public void testFindByTelephoneLike() {
		List<Emp> list = empRepository.findByTelephoneLike("%134%");
		for (Emp emp : list) {
			System.err.println(emp);
		}
	}

	@Test
	public void testQueryName() {
		List<Emp> list = empRepository.queryName("张三");
		for (Emp emp : list) {
			System.err.println(emp);
		}
	}

	@Test
	public void testQueryName2() {
		List<Emp> list = empRepository.queryName2("张三");
		for (Emp emp : list) {
			System.err.println(emp);
		}
	}

	@Test
	@Transactional // 开启事务操作 注意：@Transactional和@Test一起使用的时候，事务会自动回滚
	@Rollback(false) // 取消自动回滚
	public void testupdateAddressById() {
		empRepository.updateAddressById("广州越秀", 2);
	}

	// 添加
	@Test
	public void testSave() {
		Emp emp = new Emp();
		emp.setName("王五666");
		emp.setGender("女");
		emp.setTelephone("1346666677777");
		emp.setAddress("广州番禺");

		empRepository.save(emp);
	}

	// 修改
	@Test
	public void testUpdate() {
		Emp emp = new Emp();
		emp.setId(4);
		emp.setName("王五6666");
		emp.setGender("女");
		emp.setTelephone("1346666677777");
		emp.setAddress("广州白云");

		empRepository.save(emp);
	}

	// 查询所有数据
	@Test
	public void testFindAll() {
		// 强制转换
		List<Emp> list = (List<Emp>) empRepository.findAll();
		for (Emp emp : list) {
			System.out.println(emp);
		}
	}

	// 查询一个对象
	@Test
	public void testFineOne() {
		// 强制转换
		Emp emp = empRepository.findOne(4);
		System.out.println(emp);
	}

	// 删除一个对象
	@Test
	public void testDelete() {
		empRepository.delete(4);
	}

	// 排序
	@Test
	public void testSort() {
		// 封装排序条件的对象
		Sort sort = new Sort(new Order(Direction.DESC, "id"));

		Iterable<Emp> list = empRepository.findAll(sort);
		for (Emp emp : list) {
			System.out.println(emp);
		}
	}

	// 分页
	@Test
	public void testPagealbe() {
		// Pageable：用于封装分页参数 。 当前页码和查询记录数（注意：当前页码从0开始的）
		Pageable pageable = new PageRequest(1, 2);
		// 封装排序条件的对象
		// Page: 用于封装分页查询后的结果
		Page<Emp> pageData = empRepository.findAll(pageable);

		System.out.println("总记录数：" + pageData.getTotalElements());
		List<Emp> content = pageData.getContent();
		for (Emp emp : content) {
			System.out.println(emp);
		}
		System.out.println("总页数：" + pageData.getTotalPages());
	}

	// 排序+ 分页
	@Test
	public void testSortAndPagealbe() {
		// Pageable：用于封装分页参数 。 当前页码和查询记录数（注意：当前页码从0开始的）
		Sort sort = new Sort(new Order(Direction.DESC, "id"));

		Pageable pageable = new PageRequest(0, 2, sort);
		// 封装排序条件的对象
		// Page: 用于封装分页查询后的结果
		Page<Emp> pageData = empRepository.findAll(pageable);

		System.out.println("总记录数：" + pageData.getTotalElements());
		List<Emp> content = pageData.getContent();
		for (Emp emp : content) {
			System.out.println(emp);
		}
		System.out.println("总页数：" + pageData.getTotalPages());
	}

	// 查询所有数据
	@Test
	public void testFindAllJpaRepository() {
		// 不需要强制转换
		List<Emp> list = empRepository.findAll();
		for (Emp emp : list) {
			System.out.println(emp);
		}
	}

	// 使用JPASpecificationExecutor接口的方法（1个条件）
	@Test
	public void testFindAllJPASpecificationExecutor1() {
		/**
		 * Specification： 用于封装条件数据的对象
		 */
		Specification<Emp> spec = new Specification<Emp>() {
			//Predicate:该对象用于封装条件
			/**
			 * Root<Emp> root ： 根对象，用于查询对象的属性
			 * CriteriaQuery<?> query： 执行普通的查询
			 * CriteriaBuilder cb: 查询条件构造器，用于完成不同条件的查询
			 */
			@Override
			public Predicate toPredicate(Root<Emp> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				// where name = ?
				/**
				 * 参数一：查询的属性（需要使用root进行查询）
				 * 参数二：条件值
				 */
				Predicate pre = cb.equal(root.get("name"),"张三");
				
				return pre;
			}
		};
		
		List<Emp> list = empRepository.findAll(spec);
		for (Emp emp : list) {
			System.out.println(emp);
		}
	}
	
	// 使用JPASpecificationExecutor接口的方法（多个条件）
	@Test
	public void testFindAllJPASpecificationExecutor2() {
		Specification<Emp> spec = new Specification<Emp>() {
			@Override
			public Predicate toPredicate(Root<Emp> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				// where name = ? and gender = ?
				List<Predicate> preList = new ArrayList<Predicate>();
				
				preList.add( cb.equal(root.get("name"), "张三") );
				preList.add( cb.equal(root.get("gender"), "男") );
				
				Predicate[] preArray = new Predicate[preList.size()];
				return cb.and(preList.toArray(preArray));
			}
		};
		
		List<Emp> list = empRepository.findAll(spec);
		for (Emp emp : list) {
			System.out.println(emp);
		}
	}
	
}
