package cn.sm1234.test;

import javax.annotation.Resource;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.serializer.Jackson2JsonRedisSerializer;
import org.springframework.data.redis.serializer.JdkSerializationRedisSerializer;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import cn.sm1234.Application;
import cn.sm1234.domain.Emp;

@RunWith(SpringJUnit4ClassRunner.class)
@SpringBootTest(classes = Application.class)
public class RedisTest {

	@Resource
	private RedisTemplate<String, Object> redisTemplate;

	// 存入字符串:StringRedisSerializer
	@Test
	public void testSet() {
		redisTemplate.opsForValue().set("name", "sm1234@qq.com");
	}

	// 取出字符串:StringRedisSerializer
	@Test
	public void testGet() {
		String name = (String) redisTemplate.opsForValue().get("name");
		System.out.println(name);
	}

	// 存入JavaBean:JdkSerializationRedisSerializer
	@Test
	public void testSetJavaBean() {
		Emp emp = new Emp();
		emp.setName("张三");
		emp.setGender("男");

		// 重新设置value序列化区
		redisTemplate.setValueSerializer(new JdkSerializationRedisSerializer());

		redisTemplate.opsForValue().set("emp", emp);
	}

	// 取出JavaBean
	@Test
	public void testGetJavaBean() {
		// 重新设置value序列化区
		redisTemplate.setValueSerializer(new JdkSerializationRedisSerializer());

		Emp emp = (Emp) redisTemplate.opsForValue().get("emp");
		System.out.println(emp);
	}

	// 以JSON格式存入JavaBean
	@Test
	public void testSetJavaBeanUserJson() {
		// 重新设置value序列化区
		redisTemplate.setValueSerializer(new Jackson2JsonRedisSerializer<>(Emp.class));

		Emp emp = new Emp();
		emp.setName("李四");
		emp.setGender("男");
		
		redisTemplate.opsForValue().set("emp_json", emp);
	}
	
	// 以JSON格式取出JavaBean
	@Test
	public void testGetJavaBeanUserJson() {
		// 重新设置value序列化区
		redisTemplate.setValueSerializer(new Jackson2JsonRedisSerializer<>(Emp.class));

		Emp emp = (Emp) redisTemplate.opsForValue().get("emp_json");
		System.out.println(emp);
	}
}
